/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: amdigReg.c,v 5.1 1999/05/04 12:37:26 matthias Exp $
 *
 */

/*
 * Defines:
 */

/* #define DEBUG_ICPCI_REG */

/*
 * Includes:
 */

#include "icpciOS.h"	/* os specific stuff */
#include "icpciExt.h"	/* offical driver interface */
#include "icpciInt.h"	/* driver internals */
#include "amdigReg.h"	/* register definitions AM-DIG */

/*
 * Globals:
 */

extern struct
icp_global_t    icp;	/* main driver structure */

#if (DEBUG_ICPCI || DEBUG_ICPCI_REG)
const char  *dg_reg[] = { "modid ",
			  "modver",
			  "clkcon",
			  "ctcon ",
			  "excon ",
			  "extime",
			  "lutcon",
			  "lutsel",
			  "hoff  ",
			  "hact  ",
			  "voff  ",
			  "vact  ",
			  "trgcon",
			  "lstat ",
			  "swtrig" 
			};
#endif /* DEBUG_ICPCI_REG */

/*
 * Prototypes:
 */

/*
 * General Read Handler for AM-DIG Registers
 */

/* 16 bit register access with mask and range checking */

static u_short
dg_read_mm (board, offset, mask, min, max)
     
  u_char  board;	/* number of IC-PCI board attached to the computer */
  u_short offset;	/* offset to register base address pointer */
  u_short mask;		/* read mask for register */
  u_short min;		/* minimal value for range checking */
  u_short max;		/* maximal value for range checking */

{
#ifdef DEBUG_ICPCI_REG
  char   dg_reg_tmp[8];
#endif

  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short),	ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			  sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  tmp_val = icp_readw (icp.map[board]->v_mod_base + offset);

  tmp_val &= mask;

#if (0)
  if (tmp_val & ~mask) icp.errno[board] = -EICP_MASK;

  if (tmp_val < min || tmp_val > max) icp.errno[board] = -EICP_RANGE;
#endif

#ifdef DEBUG_ICPCI_REG
  if (offset < DG_LUT16_OFFSET) {
    if (offset > DG_REG_SIZE) return (65535);
    strcpy (dg_reg_tmp, dg_reg[offset]);
  } else {
    if (offset < DG_LUT8_OFFSET) {
      strcpy (dg_reg_tmp, "lut16 ");
    } else {
      strcpy (dg_reg_tmp, "lut8  ");
    }
  }
  icp_printf("icpci[%d]:  dg-readw  %s, %#06X &(%#06X) >%#06X <%#06X\n",
	     board, dg_reg_tmp, tmp_val, mask, min, max);
#endif

  return (tmp_val);
}

/* 16 bit register 1 (or more) bit access */

static u_char
dg_read_ms (board, offset, mask, shift)

  u_char  board;	/* number of IC-PCI board attached to the computer */
  u_short offset;	/* offset to register base address pointer */
  u_char  shift;	/* number of bits to shift right */
  u_short mask;		/* read mask for register */
  
{
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  if (offset > DG_REG_SIZE) return (255);

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short),	ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			  sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  tmp_val = icp_readw (icp.map[board]->v_mod_base + offset);

  tmp_val &= mask;
  tmp_val >>= shift;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  dg-readw  %s, %#1X &(%#06X) >>%2d\n",
	     board, dg_reg[offset], tmp_val, mask, shift);
#endif

  return ((u_char)tmp_val);
}
  
/*
 * General Write Handler for AM-DIG Registers
 */

/* 16 bit register access with mask and range checking */

static int
dg_write_mm (board, offset, mask, min, max, value)

  u_char  board;	/* number of IC-PCI board attached to the computer */
  u_short offset;	/* offset to register base address pointer */
  u_short mask;		/* read mask for register */
  u_short min;		/* minimal value for range checking */
  u_short max;		/* maximal value for range checking */
  u_short value;	/* value to set to */

{
#ifdef DEBUG_ICPCI_REG
  char	 dg_reg_tmp[8];
#endif

  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  if (value & ~mask) {
    icp.errno[board] = -EICP_MASK;
    return (ERROR);
  }

  if (value < min || value > max) {
    icp.errno[board] = -EICP_RANGE;  
    return (ERROR);
  }

  tmp_val = value & mask;

#ifdef DEBUG_ICPCI_REG
  if (offset < DG_LUT16_OFFSET) {
    if (offset > DG_REG_SIZE) {
      icp.errno[board] = -EICP_NOSUPPORT;  
      return (ERROR);
    }
    strcpy (dg_reg_tmp, dg_reg[offset]);
  } else {
    if (offset < DG_LUT8_OFFSET) {
      strcpy (dg_reg_tmp, "lut16 ");
    } else {
      strcpy (dg_reg_tmp, "lut8  ");
    }
  }
  icp_printf("icpci[%d]:  dg-writew %s, %#06X &(%#06X) >%#06X <%#06X\n",
	     board, dg_reg_tmp, tmp_val, mask, min, max);
#endif

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short),	ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			  sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  icp_writew (tmp_val, icp.map[board]->v_mod_base + offset);

  return (OK);
}

/* 16 bit register 1 (or more) bit access */

static int
dg_write_ms (board, offset, mask, shift, value)

  u_char  board;	/* number of IC-PCI board attached to the computer */
  u_short offset;	/* offset to register base address pointer */
  u_short mask;		/* read mask for register */
  u_char  shift;	/* number of bits to shift right */
  u_char  value;	/* value to set to */  
  
{
  u_short read_val = 0;
  u_short write_val = 0;
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  if (offset > DG_REG_SIZE) {
    icp.errno[board] = -EICP_NOSUPPORT;    
    return (ERROR);
  }

  tmp_val = (u_short)value << shift;

  if (tmp_val & ~mask) {
    icp.errno[board] = -EICP_MASK;
    return (ERROR);
  }

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short),	ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			  sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  read_val = icp_readw (icp.map[board]->v_mod_base + offset);

  read_val &= ~mask;
  write_val = read_val | tmp_val;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  dg-writew %s, %#06X->%#06X %#1X &(%#06X) <<%2d\n",
	     board, dg_reg[offset], read_val, write_val, tmp_val, mask, shift);
#endif

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short),	ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset / 
			  sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  icp_writew (write_val, icp.map[board]->v_mod_base + offset);

  return (OK);
}
  
/*
 * Special Read Handler for AM-DIG Registers
 */

u_char
dg_get_rev (u_char board) {

  return (dg_read_ms (board,
		      DG_MODID_OFFSET,
		      DG_REV_MASK,
		      DG_REV_SHIFT));
}

u_char
dg_get_modver (u_char board) {

  u_short tmp_val = 0;
  
  tmp_val = dg_read_mm (board,
			DG_MODVER_OFFSET,
			DG_MODVER_MASK,
			DG_MODVER_MIN,
			DG_MODVER_MAX);

#if(0)
  if (tmp_val > 4) icp.errno[board] = -EICP_RANGE;
#endif

  return ((u_char)tmp_val);
}

u_char
dg_get_fsel (u_char board) {

  u_char tmp_val = 0;
  
  tmp_val = dg_read_ms (board,
			DG_CLKCON_OFFSET,
			DG_FSEL_MASK,
			DG_FSEL_SHIFT);

#if(0)
  switch (tmp_val) {
  case 2:
  case 12:
  case 14:
  case 15:
    icp.errno[board] = -EICP_RANGE;	
    break;
  }
#endif
  
  return (tmp_val);
}

u_char
dg_get_clkdiv (u_char board) {

  return (dg_read_ms (board,
		      DG_CLKCON_OFFSET,
		      DG_CLKDIV_MASK,
		      DG_CLKDIV_SHIFT));
}

u_char
dg_get_xillen (u_char board) {

  return (dg_read_ms (board,
		      DG_CLKCON_OFFSET,
		      DG_XILLEN_MASK,
		      DG_XILLEN_SHIFT));
}

u_char
dg_get_mc (u_char board) {

  return (dg_read_ms (board,
		      DG_CLKCON_OFFSET,
		      DG_MC_MASK,
		      DG_MC_SHIFT));
}

u_char
dg_get_pclkpol (u_char board) {

  return (dg_read_ms (board,
		      DG_CTCON_OFFSET,
		      DG_PCLKPOL_MASK,
		      DG_PCLKPOL_SHIFT));
}

u_char
dg_get_lenpol (u_char board) {

  return (dg_read_ms (board,
		      DG_CTCON_OFFSET,
		      DG_LENPOL_MASK,
		      DG_LENPOL_SHIFT));
}

u_char
dg_get_fenpol (u_char board) {

  return (dg_read_ms (board,
		      DG_CTCON_OFFSET,
		      DG_FENPOL_MASK,
		      DG_FENPOL_SHIFT));
}

u_char
dg_get_fldpol (u_char board) {

  return (dg_read_ms (board,
		      DG_CTCON_OFFSET,
		      DG_FLDPOL_MASK,
		      DG_FLDPOL_SHIFT));
}

u_char
dg_get_smode (u_char board) {

  return (dg_read_ms (board,
		      DG_CTCON_OFFSET,
		      DG_SMODE_MASK,
		      DG_SMODE_SHIFT));
}

u_char
dg_get_ilmode (u_char board) {
  
  return (dg_read_ms (board,
		      DG_CTCON_OFFSET,
		      DG_ILMODE_MASK,
		      DG_ILMODE_SHIFT));
}

u_char
dg_get_lensm (u_char board) {

  return (dg_read_ms (board,
		      DG_CTCON_OFFSET,
		      DG_LENSM_MASK,
		      DG_LENSM_SHIFT));
}

u_char
dg_get_nolmiss (u_char board) {

  u_char tmp_val;

  tmp_val =  (dg_read_ms (board,
			  DG_CTCON_OFFSET,
			  DG_NOLMISS_MASK,
			  DG_NOLMISS_SHIFT));

#if(0)
  if (tmp_val != DG_ENABLE) icp.errno[board] = -EICP_NOSUPPORT;
#endif

  return (tmp_val);
}

u_char
dg_get_lvar (u_char board) {

  return (dg_read_ms (board,
		      DG_CTCON_OFFSET,
		      DG_LVAR_MASK,
		      DG_LVAR_SHIFT));
}

u_char
dg_get_exclk (u_char board) {

  u_char tmp_val = 0;

  tmp_val = dg_read_ms (board,
			DG_EXCON_OFFSET,
			DG_EXCLK_MASK,
			DG_EXCLK_SHIFT);
  
#if(0)
  if (tmp_val > 5) icp_error[board] = -EICP_RANGE;
#endif

  return (tmp_val);
}

u_char
dg_get_extmd (u_char board) {

  return (dg_read_ms (board,
		      DG_EXCON_OFFSET,
		      DG_EXTMD_MASK,
		      DG_EXTMD_SHIFT));
}

u_char
dg_get_exsen (u_char board) {

  return (dg_read_ms (board,
		      DG_EXCON_OFFSET,
		      DG_EXSEN_MASK,
		      DG_EXSEN_SHIFT));
}

u_char
dg_get_exspol (u_char board) {

  return (dg_read_ms (board,
		      DG_EXCON_OFFSET,
		      DG_EXSPOL_MASK,
		      DG_EXSPOL_SHIFT));
}

u_char
dg_get_prien (u_char board) {

  return (dg_read_ms (board,
		      DG_EXCON_OFFSET,
		      DG_PRIEN_MASK,
		      DG_PRIEN_SHIFT));
}

u_char
dg_get_prip (u_char board) {

  return (dg_read_ms (board,
		      DG_EXCON_OFFSET,
		      DG_PRIP_MASK,
		      DG_PRIP_SHIFT));
}

u_short
dg_get_extime (u_char board) {

  u_short tmp_val = 0; 

  tmp_val = dg_read_mm (board,
			DG_EXTIME_OFFSET,
			DG_EXTIME_MASK,
			DG_EXTIME_MIN,
			DG_EXTIME_MAX);
  
  return (tmp_val + 3);

}

u_char
dg_get_psize (u_char board) {

  return (dg_read_ms (board,
		      DG_LUTCON_OFFSET,
		      DG_PSIZE_MASK,
		      DG_PSIZE_SHIFT));
}

u_char
dg_get_ilpixel (u_char board) {
  
  return (dg_read_ms (board,
		      DG_LUTCON_OFFSET,
		      DG_ILPIXEL_MASK,
		      DG_ILPIXEL_SHIFT));
}

u_char
dg_get_muxa (u_char board) {

  return (dg_read_ms (board,
		      DG_LUTCON_OFFSET,
		      DG_MUXA_MASK,
		      DG_MUXA_SHIFT));
}

u_char
dg_get_muxb (u_char board) {

  return (dg_read_ms (board,
		      DG_LUTCON_OFFSET,
		      DG_MUXB_MASK,
		      DG_MUXB_SHIFT));
}

u_char
dg_get_muxc (u_char board) {

#if(0)
  if (dg_get_rev (board) < 2 || dg_get_modver (board) == DG_MODVER_24_TTL)
    icp.errno[board] = -EICP_NOSUPPORT;
#endif

  return (dg_read_ms (board,
		      DG_LUTCON_OFFSET,
		      DG_MUXC_MASK,
		      DG_MUXC_SHIFT));
}

u_char
dg_get_lutpg (u_char board) {

  u_char tmp_val;

#if(0)
  u_char mod_ver;

  mod_ver = dg_get_modver (board);
  if (modver == DG_MODVER_8_TTL || modver == DG_MODVER_8_RS422) {
    icp.errno[board] = -EICP_NOSUPPORT;
#endif

  tmp_val = dg_read_ms (board,
			DG_LUTSEL_OFFSET,
			DG_LUTPG_MASK,
			DG_LUTPG_SHIFT);

#if(0)
  switch (dg_get_psize (board)) {
  case DG_PSIZE_16:
    if (tmp_val & (u_char)~DG_PSIZE_16_MASK) {
      icp.errno[board] = -EICP_RANGE;
    }
    break;
  case DG_PSIZE_12:
    if (tmp_val & (u_char)~DG_PSIZE_12_MASK) {
      icp.errno[board] = -EICP_RANGE;
    }
    break;
  case DG_PSIZE_10:
    if (tmp_val & (u_char)~DG_PSIZE_10_MASK) {
      icp.errno[board] = -EICP_RANGE;
    }
    break;
  case DG_PSIZE_8:
    if (tmp_val & (u_char)~DG_PSIZE_8_MASK) {
      icp.errno[board] = -EICP_RANGE;
    }
    break;
  }
#endif
      
  return (tmp_val);
}

u_char
dg_get_olutpg (u_char board) {

#if(0)
  if (dg_get_modver (board) != DG_MODVER_24_TTL)
    icp.errno[board] = -EICP_NOSUPPORT;
#endif


  return (dg_read_ms (board,
		      DG_LUTSEL_OFFSET,
		      DG_OLUTPG_MASK,
		      DG_OLUTPG_SHIFT));
}

u_short
dg_get_hoff (u_char board) {

  return (dg_read_mm (board,
		      DG_HOFF_OFFSET,
		      DG_HOFF_MASK,
		      DG_HOFF_MIN,
		      DG_HOFF_MAX));
}

u_short
dg_get_hact (u_char board) {

  u_short tmp_val = 0; 

  tmp_val = dg_read_mm (board,
			DG_HACT_OFFSET,
			DG_HACT_MASK,
			DG_HACT_MIN,
			DG_HACT_MAX);
  
  return (tmp_val + 4);
}

u_short
dg_get_voff (u_char board) {

  return (dg_read_mm (board,
		      DG_VOFF_OFFSET,
		      DG_VOFF_MASK,
		      DG_VOFF_MIN,
		      DG_VOFF_MAX));
}

u_short
dg_get_vact (u_char board) {

  u_short tmp_val = 0; 

  tmp_val = dg_read_mm (board,
		      DG_VACT_OFFSET,
		      DG_VACT_MASK,
		      DG_VACT_MIN,
		      DG_VACT_MAX);

  return (tmp_val + 1);
}

u_char
dg_get_trigen (u_char board) {

  return (dg_read_ms (board,
		      DG_TRGCON_OFFSET,
		      DG_TRIGEN_MASK,
		      DG_TRIGEN_SHIFT));
}

u_char
dg_get_trigsrc (u_char board) {

  return (dg_read_ms (board,
		      DG_TRGCON_OFFSET,
		      DG_TRIGSRC_MASK,
		      DG_TRIGSRC_SHIFT));
}

u_char
dg_get_trigcy (u_char board) {

  return (dg_read_ms (board,
		      DG_TRGCON_OFFSET,
		      DG_TRIGCY_MASK,
		      DG_TRIGCY_SHIFT));
}

u_char
dg_get_vb (u_char board) {

  return (dg_read_ms (board,
		      DG_TRGCON_OFFSET,
		      DG_VBSTAT_MASK,
		      DG_VBSTAT_SHIFT));
}

u_char
dg_get_fld (u_char board) {

  return (dg_read_ms (board,
		      DG_TRGCON_OFFSET,
		      DG_FLD_MASK,
		      DG_FLD_SHIFT));
}

u_short
dg_get_lstat (u_char board) {

  return (dg_read_mm (board,
		      DG_LSTAT_OFFSET,
		      DG_LSTAT_MASK,
		      DG_LSTAT_MIN,
		      DG_LSTAT_MAX));
}

u_char
dg_get_lut16 (u_char board,  union dg_lut16_t *get_lut) {

  int 	count = 0;

  int	bank = 0,
    	banks = 0;

  u_short page = 0,
	 shift = 0;

  u_short data = 0;

#if(0)
  u_char mod_ver;

  mod_ver = dg_get_modver (board);
  if (modver == DG_MODVER_8_TTL || modver == DG_MODVER_8_RS422) {
    icp.errno[board] = -EICP_NOSUPPORT;
  }
  if (get_lut == NULL) icp.errno[board] = -EICP_POINTER;
#endif

  switch (dg_get_psize (board)) {
  case DG_PSIZE_16:
    banks = (u_char)~DG_PSIZE_16_MASK + 1;
    shift = DG_PSIZE_16_SHIFT;
    break;
  case DG_PSIZE_12:
    banks = (u_char)~DG_PSIZE_12_MASK + 1;
    shift = DG_PSIZE_12_SHIFT;
    break;
  case DG_PSIZE_10:
    banks = (u_char)~DG_PSIZE_10_MASK + 1;
    shift = DG_PSIZE_10_SHIFT;
    break;
  case DG_PSIZE_8:
    banks = (u_char)~DG_PSIZE_8_MASK + 1;
    shift = DG_PSIZE_8_SHIFT;
    break;
  }

  page = dg_get_lutpg (board);

  for (bank = 0; bank < banks; bank++) {

    dg_write_ms (board,
		 DG_LUTSEL_OFFSET,
		 DG_LUTPG_MASK,
		 DG_LUTPG_SHIFT,
		 (page << shift) + bank);
    
    for (count = 0; count < DG_LUT16_SIZE; count++) {

      data = dg_read_mm (board,
			 DG_LUT16_OFFSET + count,
			 DG_LUT16_MASK,
			 DG_LUT16_MIN,
			 DG_LUT16_MAX);
      
      switch (shift) {
      case DG_PSIZE_16_SHIFT:
	get_lut->lut16data[count + bank * DG_LUT16_SIZE] = data;
	break;
      case DG_PSIZE_12_SHIFT:
	get_lut->lut12data[count + bank * DG_LUT16_SIZE] = data;
	break;      
      case DG_PSIZE_10_SHIFT:
	get_lut->lut10data[count + bank * DG_LUT16_SIZE] = data;
	break;
      case DG_PSIZE_8_SHIFT:
	get_lut->lut8data[count + bank * DG_LUT16_SIZE] = data;
	break;
      }
    }
  }

  if ((dg_set_lutpg (board, page)) < 0) return (ERROR);

  return (OK);
}

u_char
dg_get_lut8 (u_char board, struct dg_lut8_t *get_lut) {

  int count;

#if(0)
  if (dg_get_modver (board) != DG_MODVER_24_TTL)
    icp.errno[board] = -EICP_NOSUPPORT;
  if (get_lut == NULL) icp.errno[board] = -EICP_POINTER;
#endif

  for (count = 0; count < DG_LUT8_SIZE; count++) {

    get_lut->lutdata[count] = dg_read_mm (board,
					  DG_LUT8_OFFSET + count,
					  DG_LUT8_MASK,
					  DG_LUT8_MIN,
					  DG_LUT8_MAX);
  }

  return (OK);
}
			
/*
 * Special Write Handler for AM-DIG Registers
 */

int
dg_set_fsel (u_char board, u_char value) {

    switch (value) {
    case 2:
    case 12:
    case 14:
    case 15:
      icp.errno[board] = -EICP_RANGE;
      return (ERROR);
      break;
    }	

    return (dg_write_ms (board,
			 DG_CLKCON_OFFSET,
			 DG_FSEL_MASK,
			 DG_FSEL_SHIFT,
			 value));
}

int
dg_set_clkdiv (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_CLKCON_OFFSET,
		       DG_CLKDIV_MASK,
		       DG_CLKDIV_SHIFT,
		       value));
}

int
dg_set_xillen (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_CLKCON_OFFSET,
		       DG_XILLEN_MASK,
		       DG_XILLEN_SHIFT,
		       value));
}

int
dg_set_mc (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_CLKCON_OFFSET,
		       DG_MC_MASK,
		       DG_MC_SHIFT,
		       value));
}

int
dg_set_pclkpol (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_CTCON_OFFSET,
		       DG_PCLKPOL_MASK,
		       DG_PCLKPOL_SHIFT,
		       value));
}

int
dg_set_lenpol (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_CTCON_OFFSET,
		       DG_LENPOL_MASK,
		       DG_LENPOL_SHIFT,
		       value));
}

int
dg_set_fenpol (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_CTCON_OFFSET,
		       DG_FENPOL_MASK,
		       DG_FENPOL_SHIFT,
		       value));
}

int
dg_set_fldpol (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_CTCON_OFFSET,
		       DG_FLDPOL_MASK,
		       DG_FLDPOL_SHIFT,
		       value));
}

int
dg_set_smode (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_CTCON_OFFSET,
		       DG_SMODE_MASK,
		       DG_SMODE_SHIFT,
		       value));
}

int
dg_set_ilmode (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_CTCON_OFFSET,
		       DG_ILMODE_MASK,
		       DG_ILMODE_SHIFT,
		       value));
}

int
dg_set_lensm (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_CTCON_OFFSET,
		       DG_LENSM_MASK,
		       DG_LENSM_SHIFT,
		       value));
}

int
dg_set_nolmiss (u_char board, u_char value) {

#if (0)
  if (value != DG_ENABLE) {
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }
#endif

  return (dg_write_ms (board,
		       DG_CTCON_OFFSET,
		       DG_NOLMISS_MASK,
		       DG_NOLMISS_SHIFT,
		       value));
}

int
dg_set_lvar (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_CTCON_OFFSET,
		       DG_LVAR_MASK,
		       DG_LVAR_SHIFT,
		       value));
}

int
dg_set_exclk (u_char board, u_char value) {

  if (value > 5) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }

  return (dg_write_ms (board,
		       DG_EXCON_OFFSET,
		       DG_EXCLK_MASK,
		       DG_EXCLK_SHIFT,
		       value));
}

int
dg_set_extmd (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_EXCON_OFFSET,
		       DG_EXTMD_MASK,
		       DG_EXTMD_SHIFT,
		       value));
}

int
dg_set_exsen (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_EXCON_OFFSET,
		       DG_EXSEN_MASK,
		       DG_EXSEN_SHIFT,
		       value));
}

int
dg_set_exspol (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_EXCON_OFFSET,
		       DG_EXSPOL_MASK,
		       DG_EXSPOL_SHIFT,
		       value));
}

int
dg_set_prien (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_EXCON_OFFSET,
		       DG_PRIEN_MASK,
		       DG_PRIEN_SHIFT,
		       value));
}

int
dg_set_prip (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_EXCON_OFFSET,
		       DG_PRIP_MASK,
		       DG_PRIP_SHIFT,
		       value));
}

int
dg_set_extime (u_char board, u_short value) {

  if (value < 3 || value > 4098) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }

  return (dg_write_mm (board,
		       DG_EXTIME_OFFSET,
		       DG_EXTIME_MASK,
		       DG_EXTIME_MIN,
		       DG_EXTIME_MAX,
		       value - 3));
}

int
dg_set_psize (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_LUTCON_OFFSET,
		       DG_PSIZE_MASK,
		       DG_PSIZE_SHIFT,
		       value));
}

int
dg_set_ilpixel (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_LUTCON_OFFSET,
		       DG_ILPIXEL_MASK,
		       DG_ILPIXEL_SHIFT,
		       value));
}

int
dg_set_muxa (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_LUTCON_OFFSET,
		       DG_MUXA_MASK,
		       DG_MUXA_SHIFT,
		       value));
}

int
dg_set_muxb (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_LUTCON_OFFSET,
		       DG_MUXB_MASK,
		       DG_MUXB_SHIFT,
		       value));
}

int
dg_set_muxc (u_char board, u_char value) {


  if (dg_get_rev (board) < 2 || dg_get_modver (board) == DG_MODVER_24_TTL) {
    if (value == DG_ENABLE) {
      icp.errno[board] = -EICP_NOSUPPORT;
      return (ERROR);
    }
  }

  return (dg_write_ms (board,
		       DG_LUTCON_OFFSET,
		       DG_MUXC_MASK,
		       DG_MUXC_SHIFT,
		       value));
}

int
dg_set_lutpg (u_char board, u_char value) {

  u_char mod_ver;

  mod_ver = dg_get_modver (board);
  if (mod_ver == DG_MODVER_8_TTL || mod_ver == DG_MODVER_8_RS422) {
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }

  switch (dg_get_psize (board)) {
  case DG_PSIZE_16:
    if (value & (u_char)~DG_PSIZE_16_MASK) {
      icp.errno[board] = -EICP_RANGE;
      return (ERROR);
    }
    break;
  case DG_PSIZE_12:
    if (value & (u_char)~DG_PSIZE_12_MASK) {
      icp.errno[board] = -EICP_RANGE;
      return (ERROR);
    }
    break;
  case DG_PSIZE_10:
    if (value & (u_char)~DG_PSIZE_10_MASK) {
      icp.errno[board] = -EICP_RANGE;
      return (ERROR);
    }
    break;
  case DG_PSIZE_8:
    if (value & (u_char)~DG_PSIZE_8_MASK) {
      icp.errno[board] = -EICP_RANGE;
      return (ERROR);
    }
    break;
  }

  return (dg_write_ms (board,
		       DG_LUTSEL_OFFSET,
		       DG_LUTPG_MASK,
		       DG_LUTPG_SHIFT,
		       value));
}

int
dg_set_olutpg (u_char board, u_char value) {

  if (dg_get_modver (board) != DG_MODVER_24_TTL) {
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }

  return (dg_write_ms (board,
		       DG_LUTSEL_OFFSET,
		       DG_OLUTPG_MASK,
		       DG_OLUTPG_SHIFT,
		       value));
}

int
dg_set_hoff (u_char board, u_short value) {

  return (dg_write_mm (board,
		       DG_HOFF_OFFSET,
		       DG_HOFF_MASK,
		       DG_HOFF_MIN,
		       DG_HOFF_MAX,
		       value));
}

int
dg_set_hact (u_char board, u_short value) {

  return (dg_write_mm (board,
		       DG_HACT_OFFSET,
		       DG_HACT_MASK,
		       DG_HACT_MIN,
		       DG_HACT_MAX,
		       (value - 4)));
}

int
dg_set_voff (u_char board, u_short value) {

  return (dg_write_mm (board,
		       DG_VOFF_OFFSET,
		       DG_VOFF_MASK,
		       DG_VOFF_MIN,
		       DG_VOFF_MAX,
		       value));
}

int
dg_set_vact (u_char board, u_short value) {

  return (dg_write_mm (board,
		       DG_VACT_OFFSET,
		       DG_VACT_MASK,
		       DG_VACT_MIN,
		       DG_VACT_MAX,
		       value - 1));
}

int
dg_set_trigen (u_char board, u_char value) {

  if (value == DG_ENABLE) {
    if (dg_get_trigcy (board) == DG_DISABLE) {
      icp.errno[board] = -EICP_BUSY;
      return (ERROR);
    }
  }

  return (dg_write_ms (board,
		       DG_TRGCON_OFFSET,
		       DG_TRIGEN_MASK,
		       DG_TRIGEN_SHIFT,
		       value));
}

int
dg_set_trigsrc (u_char board, u_char value) {

  return (dg_write_ms (board,
		       DG_TRGCON_OFFSET,
		       DG_TRIGSRC_MASK,
		       DG_TRIGSRC_SHIFT,
		       value));
}

int
dg_set_lut16 (u_char board, union dg_lut16_t *set_lut) {

  int 	count = 0;

  int	bank = 0,
    	banks = 0;

  u_short page = 0,
    	 shift = 0;

  u_short data = 0;

  u_char mod_ver;

  mod_ver = dg_get_modver (board);
  if (mod_ver == DG_MODVER_8_TTL || mod_ver == DG_MODVER_8_RS422) {
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }

  if (set_lut == NULL) {
    icp.errno[board] = -EICP_POINTER;
    return (ERROR);
  }

  switch (dg_get_psize (board)) {
  case DG_PSIZE_16:
    banks = (u_char)~DG_PSIZE_16_MASK + 1;
    shift = DG_PSIZE_16_SHIFT;
    break;
  case DG_PSIZE_12:
    banks = (u_char)~DG_PSIZE_12_MASK + 1;
    shift = DG_PSIZE_12_SHIFT;
    break;
  case DG_PSIZE_10:
    banks = (u_char)~DG_PSIZE_10_MASK + 1;
    shift = DG_PSIZE_10_SHIFT;
    break;
  case DG_PSIZE_8:
    banks = (u_char)~DG_PSIZE_8_MASK + 1;
    shift = DG_PSIZE_8_SHIFT;
    break;
  }

  page = dg_get_lutpg (board);

  for (bank = 0; bank < banks; bank++) {

    if (dg_write_ms (board,
		     DG_LUTSEL_OFFSET,
		     DG_LUTPG_MASK,
		     DG_LUTPG_SHIFT,
		     page + bank) < 0) return (ERROR);

    for (count = 0; count < DG_LUT16_SIZE; count++) {

      switch (shift) {
      case DG_PSIZE_16_SHIFT:
	data = set_lut->lut16data[bank * DG_LUT16_SIZE + count];
	break;
      case DG_PSIZE_12_SHIFT:
	data = set_lut->lut12data[bank * DG_LUT16_SIZE + count];
	break;      
      case DG_PSIZE_10_SHIFT:
	data = set_lut->lut10data[bank * DG_LUT16_SIZE + count];
	break;
      case DG_PSIZE_8_SHIFT:
	data = set_lut->lut8data[bank * DG_LUT16_SIZE + count];
	break;
      }

      if (dg_write_mm (board,
		       DG_LUT16_OFFSET + count,
		       DG_LUT16_MASK,
		       DG_LUT16_MIN,
		       DG_LUT16_MAX,
		       data)
	  < 0) return (ERROR);

    }
  }

  if ((dg_set_lutpg (board, page)) < 0) return (ERROR);

  return (OK);
}

int
dg_set_lut8 (u_char board, struct dg_lut8_t *set_lut) {

  int count;

  if (dg_get_modver (board) != DG_MODVER_24_TTL) {
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }

  if (set_lut == NULL) {
    icp.errno[board] = -EICP_POINTER;
    return (ERROR);
  }

  for (count = 0; count < DG_LUT8_SIZE; count++) {

    if (dg_write_mm (board,
		     DG_LUT8_OFFSET + count,
		     DG_LUT8_MASK,
		     DG_LUT8_MIN,
		     DG_LUT8_MAX,
		     set_lut->lutdata[count]) < 0) return (ERROR);
  }
  
  return (OK);
}

/*
 * Special Initialization Handler for AM-DIG Registers
 */

int
dg_init_clkcon (u_char board) {

  return (dg_write_mm (board,
		       DG_CLKCON_OFFSET,
		       DG_NO_MASK,
		       DG_NO_MIN,
		       DG_NO_MAX,
		       DG_RES_INIT));
}

int
dg_init_ctcon (u_char board) {
    
  return (dg_write_mm (board,
		       DG_CTCON_OFFSET,
		       DG_NO_MASK,
		       DG_NO_MIN,
		       DG_NO_MAX,
		       DG_RES_INIT));
}

int
dg_init_excon (u_char board) {
    
    return (dg_write_mm (board,
			 DG_EXCON_OFFSET,
			 DG_NO_MASK,
			 DG_NO_MIN,
			 DG_NO_MAX,
			 DG_RES_INIT));
}

int
dg_init_lutcon (u_char board) {
    
    return (dg_write_mm (board,
			 DG_LUTCON_OFFSET,
			 DG_NO_MASK,
			 DG_NO_MIN,
			 DG_NO_MAX,
			 DG_RES_INIT));
}

int
dg_init_lutsel (u_char board) {

  return (dg_write_mm (board,
		       DG_LUTSEL_OFFSET,
		       DG_NO_MASK,
		       DG_NO_MIN,
		       DG_NO_MAX,
		       DG_RES_INIT));
}

int
dg_init_trgcon (u_char board) {

  return (dg_write_mm (board,
		       DG_TRGCON_OFFSET,
		       DG_NO_MASK,
		       DG_NO_MIN,
		       DG_NO_MAX,
		       DG_RES_INIT));
}

/* 
 * Miscellanous functions
 */
 
/* trigger per software */

int
dg_sw_trig (u_char board) {

  return (dg_write_mm (board,
		       DG_SWTRIG_OFFSET,
		       DG_SWTRIG_MASK,
		       DG_SWTRIG_MIN,
		       DG_SWTRIG_MAX,
		       DG_SWTRIG));
}

/* linearize lookup tables */

int
dg_lin_lut16 (u_char board, u_char mode) {

  int	count = 0;

  int	page = 0,
    	pages = 0,
    	bank = 0,
    	banks = 0,
    	shift = 0;

  int	lin_lut_size = 0;

  union dg_lut16_t *lin_lut;  

  u_short data = 0;

  switch (dg_get_psize (board)) {
  case DG_PSIZE_16:
    shift = DG_PSIZE_16_SHIFT;
    pages = (DG_PSIZE_16_MASK >> DG_PSIZE_16_SHIFT) + 1;
    banks = (u_char)~DG_PSIZE_16_MASK + 1;
    lin_lut_size = sizeof (lin_lut->lut16data);
    break;
  case DG_PSIZE_12:
    shift = DG_PSIZE_12_SHIFT;
    pages = (DG_PSIZE_12_MASK >> DG_PSIZE_12_SHIFT) + 1;
    banks = (u_char)~DG_PSIZE_12_MASK + 1;
    lin_lut_size = sizeof (lin_lut->lut12data);
    break;
  case DG_PSIZE_10:
    shift = DG_PSIZE_10_SHIFT;
    pages = (DG_PSIZE_10_MASK >> DG_PSIZE_10_SHIFT) + 1;
    banks = (u_char)~DG_PSIZE_10_MASK + 1;
    lin_lut_size = sizeof (lin_lut->lut10data);
    break;
  case DG_PSIZE_8:
    shift = DG_PSIZE_8_SHIFT;
    pages = (DG_PSIZE_8_MASK >> DG_PSIZE_8_SHIFT) + 1;
    banks = (u_char)~DG_PSIZE_8_MASK + 1;
    lin_lut_size = sizeof (lin_lut->lut8data);
    break;
  }

  if ((lin_lut = icp_malloc(lin_lut_size, PAGED)) == NULL) {
    icp.errno[board] = -EICP_POINTER;
    goto error;
  }

  for (bank = 0; bank < banks; bank++) {
    for (count = 0; count < DG_LUT16_SIZE; count++) {

      switch (mode) {
      case DG_MODE_LINEAR:
	data = bank * DG_LUT16_SIZE + count;
	break;
      case DG_MODE_NORM_TO8:
	data = (bank * DG_LUT16_SIZE + count) >> shift;
	break;
      case DG_MODE_NORM_TO16:
	data = (bank * DG_LUT16_SIZE + count) << (8 - shift);
	break;
      }

      switch (shift) {
      case DG_PSIZE_16_SHIFT:
	lin_lut->lut16data[bank * DG_LUT16_SIZE + count] = data;
	break;
      case DG_PSIZE_12_SHIFT:
	lin_lut->lut12data[bank * DG_LUT16_SIZE + count] = data;
	break;
      case DG_PSIZE_10_SHIFT:
	lin_lut->lut10data[bank * DG_LUT16_SIZE + count] = data;
	break;
      case DG_PSIZE_8_SHIFT:
	lin_lut->lut8data[bank * DG_LUT16_SIZE + count] = data;
	break;
      }
    }
  }

  if (dg_set_lutpg (board, 0) < 0) goto error;
  if (dg_set_lut16 (board, lin_lut) < 0) goto error;

  memset (lin_lut, 0x00, lin_lut_size);

  for (page = 1; page < pages; page++) {

    if (dg_set_lutpg (board, page << shift) < 0) goto error;
    if (dg_set_lut16 (board, lin_lut) < 0) goto error;
  }

  if (dg_set_lutpg (board, 0) < 0) goto error;

  icp_free (lin_lut, PAGED);
  return (OK);

error:
  icp_free (lin_lut, PAGED);
  icp_error (board, "dg_set_lut16");
  return (ERROR);  
}

int
dg_lin_lut8 (u_char board) {

  int	count = 0,
    	page = 0;

  struct dg_lut8_t lin_lut;  

  for (count = 0; count < DG_LUT8_SIZE; count++)
    lin_lut.lutdata[count] = count;

  if (dg_set_olutpg (board, 0) < 0) return (ERROR);
  if (dg_set_lut8 (board, &lin_lut) < 0) return (ERROR);

  memset (&lin_lut, 0x00, sizeof(struct dg_lut8_t));

  for (page = 1; page < DG_OLUTPG_MAX; page++) {

    if (dg_set_olutpg (board, page) < 0) return (ERROR);
    if (dg_set_lut8 (board, &lin_lut) < 0) return (ERROR);
  }

  if (dg_set_olutpg (board, 0) < 0) return (ERROR);

  return (OK);
}

#ifdef DEBUG_ICPCI
/* make register dump */
void
dg_dump_regs (u_char board) {
    
  int count;

  for (count = 0; count < DG_REG_SIZE; count++) {
    icp_printf ("icpci[%d]: %s %#04x\n", board, dg_reg[count],
		dg_read_mm (board, count, DG_NO_MASK, DG_NO_MIN, DG_NO_MAX));
  }
}
#endif

